==> demoniac-0.3.tar.gz: demoniac-0.3/libfx/optbool.cpp <==
/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Boolean option for demoniac
 * leto/calodox, 2001
 */

#include <fx.h>
#include "fxinstance.h"
#include "optbool.h"
#include "interpolator.h"
#include "glmain.h"
#include "report.h"

void registerBoolOpt(char *optName, int flags, bool *value){
	if (currentFX)
		currentFX->opts.push_back( (GenericOption *)
			new OptBool(currentFX, optName, flags, value ) );
}

OptBool::OptBool(FXInstance *fxi, char *name, int flags, bool *value) :
		GenericOption(fxi, name, "Flag", flags),
		toggle(0)
{

	this->bvalue = value;
	this->value.set(*value);

	if (flags & OPTION_REINIT) {
		track = 0;
	} else {
		track = new Track(&this->value, &fixedInterpolator);
	}
}

gint cbOptBool(GtkWidget *widget, gpointer data) {
	OptBool *obj = (OptBool *) data;

	if (obj->ignore)
		return TRUE;

	debug("cb bool");

	*obj->bvalue = (bool) GTK_TOGGLE_BUTTON (widget)->active;
	obj->value.set(*obj->bvalue);

	if (obj->track) {
		obj->track->setValue(glMainptr->getTimeMilliSec(), 
				&obj->value);
	}

	glMainptr->reDraw();
	return TRUE;
}

void OptBool::computeValue(int time) {
	if (track) {
		track->computeValue(time);
		*bvalue = value.asBool();
	}
}

void OptBool::draw(GtkBox *box) {
	
	ignore = false; 

	GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(box, hbox, FALSE, FALSE, 0);

	toggle = gtk_toggle_button_new_with_label(name);
	gtk_box_pack_start(GTK_BOX(hbox), toggle, TRUE, TRUE, 0);
	
	if ((flags & OPTION_REINIT) == 0) {
		gtk_signal_connect(GTK_OBJECT(toggle), "toggled",
			GTK_SIGNAL_FUNC(cbOptBool), this);
		drawInterp(GTK_BOX(hbox));
	}
	updateGui();

} 

void OptBool::update() {
	*bvalue = (bool) GTK_TOGGLE_BUTTON (toggle)->active;
}

void OptBool::updateGui() {
	if (toggle) {
		ignore = true;
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle), *bvalue);
		ignore = false;
	}
}

void OptBool::save(Saver *s) {
    s->saveObjectBegin("OptBool");

    s->saveString("name", name);
    s->saveInt("flags", flags);

    if (flags & OPTION_REINIT) {
	s->saveInt("bvalue", *bvalue);
    } else {
	track->save(s);
    }

    s->saveObjectEnd();
}

void OptBool::load(Loader *l) {
	assert(strcmp(l->nextName(), "OptBool")==0);
	l->nextLine();

	name = new char[128];
	l->getString(name, 127);
	l->nextLine();

	flags = l->getInt();
	l->nextLine();

	if (strcmp("bvalue", l->nextName())==0) {
		track = 0;
		*bvalue = (bool) l->getInt();
		l->nextLine();
	} 

	if (l->nextType() == Loader::OBJECT) {
		track = new Track(&this->value, &fixedInterpolator);
		track->load(l);
	}

	assert(l->nextType() == Loader::OBJECT_END);
	l->nextLine();
}

==> demoniac-0.4.tar.gz: demoniac-0.4/libfx/optbool.cpp <==
/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Boolean option for demoniac
 * leto/calodox, 2001
 */

#include <fx.h>
#include "fxinstance.h"
#include "optbool.h"
#include "interpolator.h"
#include "glmain.h"
#include "report.h"

void registerBoolOpt(char *optName, int flags, bool *value){
	if (currentFX)
		currentFX->opts.push_back( (GenericOption *)
			new OptBool(currentFX, optName, flags, value ) );
}

OptBool::OptBool(FXInstance *fxi, char *name, int flags, bool *value) :
		GenericOption(fxi, name, "Flag", flags),
		toggle(0)
{

	this->bvalue = value;
	this->value.set(*value);

	if (flags & OPTION_REINIT) {
		track = 0;
	} else {
		track = new Track(&this->value, &fixedInterpolator);
	}
}

gint cbOptBool(GtkWidget *widget, gpointer data) {
	OptBool *obj = (OptBool *) data;

	if (obj->ignore)
		return TRUE;

	debug("cb bool");

	*obj->bvalue = (bool) GTK_TOGGLE_BUTTON (widget)->active;
	obj->value.set(*obj->bvalue);

	if (obj->track) {
		obj->track->setValue(obj->owner->absToRel(glMainptr->getTimeMilliSec()),
				&obj->value);
	}

	glMainptr->reDraw();
	return TRUE;
}

void OptBool::computeValue(int time) {
	if (track) {
		track->computeValue(time);
		*bvalue = value.asBool();
	}
}

void OptBool::draw(GtkBox *box) {
	
	ignore = false; 

	GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(box, hbox, FALSE, FALSE, 0);

	toggle = gtk_toggle_button_new_with_label(name);
	gtk_box_pack_start(GTK_BOX(hbox), toggle, TRUE, TRUE, 0);
	
	if ((flags & OPTION_REINIT) == 0) {
		gtk_signal_connect(GTK_OBJECT(toggle), "toggled",
			GTK_SIGNAL_FUNC(cbOptBool), this);
	}
	updateGui();

} 

void OptBool::update() {
	*bvalue = (bool) GTK_TOGGLE_BUTTON (toggle)->active;
}

void OptBool::updateGui() {
	if (toggle) {
		ignore = true;
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle), *bvalue);
		ignore = false;
	}
}

void OptBool::save(Saver *s) {
    s->saveObjectBegin("OptBool");

    s->saveString("name", name);
    s->saveInt("flags", flags);

    if (flags & OPTION_REINIT) {
	s->saveInt("bvalue", *bvalue);
    } else {
	track->save(s);
    }

    s->saveObjectEnd();
}

void OptBool::load(Loader *l) {
	assert(strcmp(l->nextName(), "OptBool")==0);
	l->nextLine();

	name = new char[128];
	l->getString(name, 127);
	l->nextLine();

	flags = l->getInt();
	l->nextLine();

	if (strcmp("bvalue", l->nextName())==0) {
		track = 0;
		*bvalue = (bool) l->getInt();
		l->nextLine();
	} 

	if (l->nextType() == Loader::OBJECT) {
		track = new Track(&this->value, &fixedInterpolator);
		track->load(l);
	}

	assert(l->nextType() == Loader::OBJECT_END);
	l->nextLine();
}
